#include "data.h"

//default constructor
tdata::tdata()
{
}

/**The database as well as the query is initialized.*/
tdata::tdata(QSqlDatabase database)
{
    db = database;
    query = QSqlQuery(db);

    vehicle_speed = NULL;
    course = NULL;
    x_acceleration = NULL;
    y_acceleration = NULL;
    z_acceleration = NULL;
    x_acceleration_gyro = NULL;
    y_acceleration_gyro = NULL;
    z_acceleration_gyro = NULL;
    longitude = NULL;
    latitude = NULL;
    altitude = NULL;
    vehicle_speed_obd = NULL;
    engine_rpm_obd = NULL;
    throttle_position_obd = NULL;
    barometric_pressure_obd = NULL;
    ambient_air_temperature_obd = NULL;
    actual_engine_torque_obd = NULL;
    engine_reference_torque_obd = NULL;
    intake_air_temperature_obd = NULL;
    utc = NULL;
}

//destructor
tdata::~tdata()
{
    delete vehicle_speed;
    delete course;
    delete x_acceleration;
    delete y_acceleration;
    delete z_acceleration;
    delete x_acceleration_gyro;
    delete y_acceleration_gyro;
    delete z_acceleration_gyro;
    delete longitude;
    delete latitude;
    delete altitude;
    delete vehicle_speed_obd;
    delete engine_rpm_obd;
    delete throttle_position_obd;
    delete barometric_pressure_obd;
    delete ambient_air_temperature_obd;
    delete actual_engine_torque_obd;
    delete engine_reference_torque_obd;
    delete intake_air_temperature_obd;
    delete utc;
}

/**Load data from .sqlite file.*/
int tdata::loadFromFile(QString path, QString name0)
{
    //init
    file_path = path;
    name = name0;

    //open database
    db.setDatabaseName(path);

    if(!db.open())
    {
        QSqlError error = db.lastError();
        qDebug() << error.text();
    }

    else
        qDebug("database loaded");

    //debug
    if(db.isValid())
        qDebug() << "valid database";
    else
        qDebug() << "invalid database";

    //get length and init arrays
    query = QSqlQuery(db);
    initArrays(query);

    //read data
    loadGPSData(query);
    loadOBDData(query);
    loadSensorData(query);
    loadUTCData(query);

    //close the database
    db.close();

    return 0;
}

/**Dynamically initialize arrays with the respective length to fit all data.*/
int tdata::initArrays(QSqlQuery query)
{
    query.prepare("SELECT COUNT(UTC) FROM OBD");
    if(!query.exec())
    {
        QSqlError error = query.lastError();
        qDebug() << error.text();
    }

    //get number of data points
    query.next();
    length = query.value(0).toInt();
    query.finish();

    //get vehicle id
    query.prepare("SELECT VehicleID FROM OBD");
    if(!query.exec())
    {
        QSqlError error = query.lastError();
        qDebug() << error.text();
    }

    query.next();
    vid = query.value(0).toString();
    query.finish();

    //performance boost
    query.setForwardOnly(true);

    //delete arrays
    delete vehicle_speed;
    delete course;
    delete x_acceleration;
    delete y_acceleration;
    delete z_acceleration;
    delete x_acceleration_gyro;
    delete y_acceleration_gyro;
    delete z_acceleration_gyro;
    delete longitude;
    delete latitude;
    delete altitude;
    delete vehicle_speed_obd;
    delete engine_rpm_obd;
    delete throttle_position_obd;
    delete barometric_pressure_obd;
    delete ambient_air_temperature_obd;
    delete actual_engine_torque_obd;
    delete engine_reference_torque_obd;
    delete intake_air_temperature_obd;
    delete utc;

    //create arrays
    vehicle_speed = new double[length];
    course = new double[length];
    x_acceleration = new double[length];
    y_acceleration = new double[length];
    z_acceleration = new double[length];
    x_acceleration_gyro = new double[length];
    y_acceleration_gyro = new double[length];
    z_acceleration_gyro = new double[length];
    longitude = new double[length];
    latitude = new double[length];
    altitude = new double[length];
    vehicle_speed_obd = new double[length];
    engine_rpm_obd = new double[length];
    throttle_position_obd = new double[length];
    barometric_pressure_obd = new double[length];
    ambient_air_temperature_obd = new double[length];
    actual_engine_torque_obd = new double[length];
    engine_reference_torque_obd = new double[length];
    intake_air_temperature_obd = new double[length];
    utc = new QStringList;

    return 0;
}

/**Gets gps data from file.*/
int tdata::loadGPSData(QSqlQuery query)
{
    query.prepare("SELECT Longitude,Latitude,Altitude,"
                  "Course,Speed FROM GPS");

    if(!query.exec())
    {
        QSqlError error = query.lastError();
        qDebug() << error.text();
    }

    //read gps values
    for(int i = 0; i<length; i++)
    {
        query.next();

        longitude[i] = query.value(0).toDouble()*qPow(10,-6);
        latitude[i] = query.value(1).toDouble()*qPow(10,-6);
        altitude[i] = query.value(2).toDouble();
        course[i] = query.value(3).toDouble()*qPow(10,-2);
        vehicle_speed[i] = query.value(4).toDouble()*qPow(10,-2);
    }

    query.finish();

    return 0;
}

/**Gets obd data from file.*/
int tdata::loadOBDData(QSqlQuery query)
{
    query.prepare("SELECT EngineRPM,VehicleSpeed,ThrottlePosition,"
                  "BarometricPressure,AmbientAirTemperature,ActualEnginePercentTorque,"
                  "EngineReferenceTorque,IntakeAirTemperature FROM OBD");

    if(!query.exec())
    {
        QSqlError error = query.lastError();
        qDebug() << error.text();
    }

    //read obd values
    for(int i = 0; i<length; i++)
    {
        query.next();

        engine_rpm_obd[i] = query.value(0).toDouble();
        vehicle_speed_obd[i] = query.value(1).toDouble();
        throttle_position_obd[i] = query.value(2).toDouble();
        barometric_pressure_obd[i] = query.value(3).toDouble();
        ambient_air_temperature_obd[i] = query.value(4).toDouble();
        actual_engine_torque_obd[i] = query.value(5).toDouble();
        engine_reference_torque_obd[i] = query.value(6).toDouble();
        intake_air_temperature_obd[i] = query.value(7).toDouble();
    }

    query.finish();

    return 0;
}

/**Gets sensor data from file.*/
int tdata::loadSensorData(QSqlQuery query)
{
    query.prepare("SELECT XAcceleration,YAcceleration,ZAcceleration,"
                  "XGyroscope,YGyroscope,ZGyroscope FROM SENSOREN");

    if(!query.exec())
    {
        QSqlError error = query.lastError();
        qDebug() << error.text();
    }

    //read sensor values
    for(int i = 0; i<length; i++)
    {
        query.next();

        x_acceleration[i] = query.value(0).toDouble()*qPow(10,-2);
        y_acceleration[i] = query.value(1).toDouble()*qPow(10,-2);
        z_acceleration[i] = query.value(2).toDouble()*qPow(10,-2);
        x_acceleration_gyro[i] = query.value(3).toDouble()*qPow(10,-2);
        y_acceleration_gyro[i] = query.value(4).toDouble()*qPow(10,-2);
        z_acceleration_gyro[i] = query.value(5).toDouble()*qPow(10,-2);
    }

    query.finish();

    return 0;
}

/**Gets utc data from file.*/
int tdata::loadUTCData(QSqlQuery query)
{
    query.prepare("SELECT UTC FROM SENSOREN");

    if(!query.exec())
    {
        QSqlError error = query.lastError();
        qDebug() << error.text();
    }

    //read utc values
    for(int i = 0; i<length; i++)
    {
        query.next();

        QString time = query.value(0).toString();
        //time.remove(0,time.indexOf(" ")+1);
        utc->append(time);
    }

    query.finish();

    return 0;
}

/**Handles export of data.*/
int tdata::exportToFile(bool csv, bool pictures, int beginning, int end, QString path)
{
    if(QDir(path + "/" + name).exists())
    {
        QMessageBox *msg = new QMessageBox;
        msg->setText(QObject::tr("Data is already existing"));
        msg->setIcon(QMessageBox::Warning);
        msg->setWindowIcon(QIcon(":/images/icon.png"));
        msg->setWindowTitle(QObject::tr("Error"));
        msg->exec();
        delete msg;
        return 1;
    }

    QDir dir;
    dir.setPath(path);
    dir.mkdir(name);
    path = path + "/" + name;

    //export as .csv
    if(csv)
        exportCsv(beginning, end, path);

    //export as .sql
    else
        exportSqlite(beginning, end, path);

    //export pictures
    if(pictures)
        exportPictures(beginning, end, path);

    QMessageBox *msg = new QMessageBox;
    msg->setText(QObject::tr("Data exported successfully"));
    msg->setWindowIcon(QIcon(":/images/icon.png"));
    msg->setWindowTitle(QObject::tr("Finished"));
    msg->exec();
    delete msg;

    return 0;
}

/**Exports data as .csv.*/
int tdata::exportCsv(int beginning, int end, QString path)
{
    QDir *dir = new QDir;
    dir->setPath(path);

    QFile *file = new QFile(dir->absoluteFilePath("data.csv"));
    file->open(QIODevice::WriteOnly | QIODevice::Text);
    QTextStream out(file);

    out << "\"UTC\";\"LONGITUDE\";\"LATITUDE\";\"ALTITUDE\";\"COURSE\";"
           "\"SPEED\";\"X_ACCELERATION\";\"Y_ACCELERATION\";\"Z_ACCELERATION\";"
           "\"X_GYROSCOPE\";\"Y_GYROSCOPE\";\"Z_GYROSCOPE\";\"VEHICLE_ID\";\"ENGINE_RPM\";"
           "\"VEHICLE_SPEED\";\"THROTTLE_POSITION\";\"BAROMETRIC_PRESSURE\";"
           "\"AMBIENT_AIR_TEMPERATURE\";\"ACTUAL_ENGINE_PERCENT_TORQUE\";"
           "\"ENGINE_REFERENCE_TORQUE\";\"INTAKE_AIR_TEMPERATURE\"\n";

    //write data to file
    for(int i = beginning; i<end; i++)
    {
        out << "\"";
        out << utc->at(i);
        out << "\";\"";
        out << longitude[i];
        out << "\";\"";
        out << latitude[i];
        out << "\";\"";
        out << altitude[i];
        out << "\";";
        out << course[i];
        out << ";";
        out << vehicle_speed[i];
        out << ";";
        out << x_acceleration[i];
        out << ";";
        out << y_acceleration[i];
        out << ";";
        out << z_acceleration[i];
        out << ";";
        out << x_acceleration_gyro[i];
        out << ";";
        out << y_acceleration_gyro[i];
        out << ";";
        out << z_acceleration_gyro[i];
        out << ";";
        out << vid;
        out << ";";
        out << engine_rpm_obd[i];
        out << ";";
        out << vehicle_speed_obd[i];
        out << ";";
        out << throttle_position_obd[i];
        out << ";";
        out << barometric_pressure_obd[i];
        out << ";";
        out << ambient_air_temperature_obd[i];
        out << ";";
        out << actual_engine_torque_obd[i];
        out << ";";
        out << engine_reference_torque_obd[i];
        out << ";";
        out << intake_air_temperature_obd[i];
        out << "\n";
    }

    //close file and delete handles
    file->close();
    delete file;
    delete dir;

    return 0;
}

/**Exports data as .sqlite.*/
int tdata::exportSqlite(int beginning, int end, QString path)
{
    //copy file
    QFile *file = new QFile(file_path);
    QDir *dir = new QDir;
    dir->setPath(path);
    file->copy(dir->absoluteFilePath("data.sqlite"));

    //open db
    db.setDatabaseName(dir->absoluteFilePath("data.sqlite"));
    if(!db.open())
    {
        QSqlError error = db.lastError();
        qDebug() << error.text();
    }

    else
        qDebug() << "database opened";

    //debug
    if(db.isValid())
        qDebug() << "valid database";
    else
        qDebug() << "invalid database";

    //delete rows if necessary
    QString q;

    //delete OBD data
    q = "DELETE FROM OBD WHERE ROWID < ";
    q.append(QString::number(beginning,10));
    q.append(" OR ROWID > ");
    q.append(QString::number(end,10));
    db.exec(q);

    //delete SENSOR data
    q = "DELETE FROM SENSOREN WHERE ROWID < ";
    q.append(QString::number(beginning,10));
    q.append(" OR ROWID > ");
    q.append(QString::number(end,10));
    db.exec(q);

    //delete GPS data
    q = "DELETE FROM GPS WHERE ROWID < ";
    q.append(QString::number(beginning,10));
    q.append(" OR ROWID > ");
    q.append(QString::number(end,10));
    db.exec(q);

    db.close();

    //close file and delete handles
    file->close();
    delete file;
    delete dir;

    return 0;
}

/**Exports pictures to the given directory.*/
int tdata::exportPictures(int beginning, int end, QString path)
{
    //create image directory
    QDir dir;
    dir.setPath(path);
    dir.mkdir("DCIM");
    dir.cd("DCIM");

    //get picture info and sort them by time
    QDir sourcedir;
    sourcedir.setPath(QDir::toNativeSeparators(QDir::homePath()+"/itlogger/"+name+"/DCIM"));
    sourcedir.setSorting(QDir::Time);
    QFileInfoList infolist = sourcedir.entryInfoList(QDir::Files);

    //get beginning and end time of export
    QDateTime time = QDateTime::fromString(utc->at(beginning),"yyyy-MM-dd HH:mm:ss:zzz");
    qint64 beginning_time = time.toMSecsSinceEpoch();
    time = QDateTime::fromString(utc->at(end-1),"yyyy-MM-dd HH:mm:ss:zzz");
    qint64 end_time = time.toMSecsSinceEpoch();

    //copy each file if it's .jpg and inside the export time window
    Q_FOREACH(QFileInfo info, infolist)
    {
        QString filename = info.fileName();
        filename.remove(0,4);
        filename.remove(filename.lastIndexOf("."),filename.count()-filename.lastIndexOf("."));
        QDateTime time0 = QDateTime::fromString(filename,"yyyyMMdd_HHmmss");
        qint64 filetime = time0.toMSecsSinceEpoch();

        qDebug() << info.suffix();

        if(filetime <= end_time && filetime >= beginning_time && info.suffix() == "jpg")
            QFile::copy(info.absoluteFilePath(),QDir::toNativeSeparators(dir.absolutePath()+"/"+info.fileName()));
    }

    return 0;
}
