#include "map.h"

map::map(QGeoMappingManager *manager) : QGraphicsGeoMap(manager)
{
}

/**Initializes the map.*/
int map::init(QGraphicsView *gui_view)
{
    //add to scene
    scene = new QGraphicsScene;
    scene->addItem(this);

    //init view
    view = gui_view;
    view->setScene(scene);
    view->setVisible(true);
    view->setInteractive(true);

    //set standard viewport to tu-ilmena
    this->setZoomLevel(14);
    this->setCenter(QGeoCoordinate(50.681341, 10.922745));

    //init polyline and panning
    polyline = new QGeoMapPolylineObject;
    pan_active = false;

    //init the map markers
    startmarker = new QGeoMapPixmapObject;
    endmarker = new QGeoMapPixmapObject;
    carmarker = new QGeoMapPixmapObject;

    QPoint offset = QPoint(-15, -36);
    QPoint offset_car = QPoint(-8,-8);

    endmarker->setOffset(offset);
    startmarker->setOffset(offset);
    carmarker->setOffset(offset_car);

    startmarker->setZValue(1);
    endmarker->setZValue(2);
    carmarker->setZValue(3);

    startmarker->setPixmap(QPixmap(":/images/startmarker.png").scaledToWidth(30, Qt::SmoothTransformation));
    endmarker->setPixmap(QPixmap(":/images/endmarker.png").scaledToWidth(30, Qt::SmoothTransformation));
    carmarker->setPixmap(QPixmap(":/images/obd.png"));

    addMapObject(startmarker);
    addMapObject(endmarker);

    car_added = false;

    //event filter for interaction
    view->installEventFilter(this);

    return 0;
}

/**Sets the route to display on the map.*/
int map::setRoute(double *longitude, double *latitude, int beginning, int end)
{
    //if route exists already, remove it
    if(polyline != NULL)
        this->removeMapObject(polyline);

    //clear old coordinates
    coordinate_list.clear();

    //accumulate new coordinate list
    for(int i = beginning; i<end; i++)
    {
        QGeoCoordinate coord;
        coord.setLatitude(latitude[i]);
        coord.setLongitude(longitude[i]);

        coordinate_list.append(coord);
    }

    //init new polyline
    polyline->setZValue(0);
    polyline->setPath(coordinate_list);
    QColor routeColor(Qt::blue);
    routeColor.setAlpha(127);
    QPen pen(routeColor);
    pen.setWidth(4);
    pen.setCapStyle(Qt::RoundCap);
    polyline->setPen(pen);
    this->addMapObject(polyline);

    //fit in viewport
    QGeoBoundingBox box = polyline->boundingBox();
    if(box.width() > 0 && box.height() > 0)
        this->fitInViewport(polyline->boundingBox());

    //apply start-/endmarkers
    setMarkers(longitude[beginning],latitude[beginning],coordinate_list.last().longitude(),coordinate_list.last().latitude());
    return 0;
}

/**Places the start-/endmarker on the map.*/
int map::setMarkers(double start_longitude, double start_latitude, double end_longitude, double end_latitude)
{
    //set coordinates
    startmarker->setCoordinate(QGeoCoordinate(start_latitude,start_longitude));
    endmarker->setCoordinate(QGeoCoordinate(end_latitude,end_longitude));
    return 0;
}

/**Places the car on the map at the given location.*/
int map::setCar(double longitude, double latitude)
{
    //add car to map if it's not already added
    if(!car_added)
    {
        addMapObject(carmarker);
        car_added = true;
    }

    //set coordinates, center and zoom on the car
    carmarker->setCoordinate(QGeoCoordinate(latitude,longitude));
    this->setCenter(QGeoCoordinate(latitude,longitude));
    this->setZoomLevel(12);
    return 0;
}

/**Removes the car from the map.*/
int map::removeCar()
{
    //remove the car if it's added
    if(car_added)
    {
        removeMapObject(carmarker);
        car_added = false;
    }

    return 0;
}

/**Activates panning.*/
void map::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    //activate panning and set hand cursor
    pan_active = true;
    this->setCursor(Qt::ClosedHandCursor);
    event->accept();
}

/**Deactivates panning.*/
void map::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    //deactivate panning
    pan_active = false;
    this->setCursor(Qt::OpenHandCursor);
    event->accept();
}

/**Pan the map.*/
void map::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    //pan the map to where the mouse goes
    if(pan_active)
    {
        QPointF delta = event->lastPos() - event->pos();
        pan(delta.x(), delta.y());
    }
    event->accept();
}

/**Zoom the map.*/
void map::wheelEvent(QGraphicsSceneWheelEvent *event)
{
    //zoom++
    qreal panx = event->pos().x() - size().width() / 2.0;
    qreal pany = event->pos().y() - size().height() / 2.0;
    pan(panx, pany);
    if(event->delta() > 0)
    {
        if(zoomLevel() < maximumZoomLevel())
        {
            setZoomLevel(zoomLevel() + 1);
        }
    }

    //zoom--
    else
    {
        if(zoomLevel() > minimumZoomLevel())
        {
            setZoomLevel(zoomLevel() - 1);
        }
    }
    pan(-panx, -pany);
    event->accept();
}

/**Filter keypress events to disable QGraphicView's panning feature.*/
bool map::eventFilter(QObject *obj, QEvent *event)
{
    if(obj == view && event->type() == QEvent::Resize)
        this->resize(view->width(),view->height());

    if(obj == view && event->type() == QEvent::KeyPress)
        return true;

    return false;
}
