#include "plot.h"

/**Initializes the plotter, sets the structure and appearance.*/
plot::plot(QwtPlot *plotter_ui, int curves, bool colored, QString yaxis_title,
           QString curve_title0, QString curve_title1, QString curve_title2, bool duo)
{
    //set pointer to ui widget
    plotter = plotter_ui;

    //axis title
    plotter->setAxisTitle(2, tr("Universal Time Coordinated [h:m:s:ms]"));

    //initialize curves
    for(int i = 0; i<curves; i++)
    {
        curve[i] = new QwtPlotCurve;
        pen[i] = new QPen(Qt::blue,2);
        curve[i]->setPen(*pen[i]);
        curve[i]->setTitle("debug");

        curve[i]->setRenderHint(QwtPlotItem::RenderAntialiased);
        curve[i]->setLegendAttribute(QwtPlotCurve::LegendShowLine);
        curve[i]->setYAxis(QwtPlot::yLeft);

        //add curves to plotter
        curve[i]->attach(plotter);
    }

    //set plotter canvas and grid
    plotter->setCanvasBackground(QBrush(Qt::white, Qt::SolidPattern));
    plotter->canvas()->setBorderRadius(4);
    grid = new QwtPlotGrid;
    grid->setPen(QPen(Qt::black,0.0,Qt::DashLine));
    grid->attach(plotter);

    //initialize zoomer
    zoomer = new QwtPlotZoomer(plotter->canvas(),true);
    zoomer->setMousePattern(QwtEventPattern::MouseSelect2,Qt::RightButton, Qt::ControlModifier);
    zoomer->setMousePattern(QwtEventPattern::MouseSelect3,Qt::RightButton);
    zoomer->setRubberBandPen(QColor(Qt::black));
    zoomer->setTrackerMode(QwtPicker::AlwaysOff);

    //initialize picker
    picker = new CustomPicker(plotter->canvas(), duo);

    //initialize panner
    panner = new QwtPlotPanner(plotter->canvas());
    panner->setMouseButton(Qt::MidButton);

    //initialize legend
    legend = new QwtLegend(0);
    legend->setFrameStyle(QFrame::NoFrame);
    plotter->insertLegend(legend, QwtPlot::BottomLegend);

    //set color
    if(colored)
    {
        int j = 0;
        for(int i = 0; i<curves; i++)
        {
            if(j == 2)
            {
                pen[i]->setColor(Qt::green);
                curve[i]->setPen(*pen[i]);
                j = 0;
                continue;
            }

            if(j == 1)
            {
                pen[i]->setColor(Qt::red);
                curve[i]->setPen(*pen[i]);
                j = 2;
                continue;
            }

            if(j == 0)
            {
                pen[i]->setColor(Qt::blue);
                curve[i]->setPen(*pen[i]);
                j = 1;
                continue;
            }
        }
    }

    //set titles
    setYaxisTitle(yaxis_title);

    setCurveTitle(curve_title0,0);
    setCurveTitle(curve_title1,1);
    setCurveTitle(curve_title2,2);

    //ensures that the rightmost x-label is visible
    QwtScaleWidget *scaleWidget = plotter->axisWidget(QwtPlot::xBottom);
    const int fmh = QFontMetrics(scaleWidget->font()).height();
    scaleWidget->setMinBorderDist(0, fmh / 2);

    //connect signal for zooming
    connect(zoomer,SIGNAL(zoomed(QRectF)),this,SLOT(on_zoomer_zoomed()));

    //axis label rotation
    plotter->setAxisLabelRotation(QwtPlot::xBottom, -45);
    plotter->setAxisLabelAlignment(QwtPlot::xBottom, Qt::AlignLeft | Qt::AlignBottom);

    //set time scale
    setTimeScale();

    //done
    plotter->replot();
}

/**Applys data to the curves and displays it.*/
int plot::setData(double *xdata, double *ydata, int curve_id, int length)
{
    //set raw samples, reset zoom and replot
    curve[curve_id]->setRawSamples(xdata,ydata,length);
    resetZoom();
    plotter->replot();
    return 0;
}

/**Sets the color of the curves.*/
int plot::setColor(int color, int curve_id)
{
    //apply colors according to user choice
    switch(color)
    {
        case 0:
            pen[curve_id]->setColor(Qt::blue);
            curve[curve_id]->setPen(*pen[curve_id]);
            plotter->replot();
            break;
        case 1:
            pen[curve_id]->setColor(Qt::red);
            curve[curve_id]->setPen(*pen[curve_id]);
            plotter->replot();
            break;
        case 2:
            pen[curve_id]->setColor(Qt::green);
            curve[curve_id]->setPen(*pen[curve_id]);
            plotter->replot();
            break;
        case 3:
            pen[curve_id]->setColor(Qt::yellow);
            curve[curve_id]->setPen(*pen[curve_id]);
            plotter->replot();
            break;
        case 4:
            pen[curve_id]->setColor(Qt::cyan);
            curve[curve_id]->setPen(*pen[curve_id]);
            plotter->replot();
            break;
        case 5:
            pen[curve_id]->setColor(Qt::magenta);
            curve[curve_id]->setPen(*pen[curve_id]);
            plotter->replot();
            break;
        case 6:
            pen[curve_id]->setColor(Qt::gray);
            curve[curve_id]->setPen(*pen[curve_id]);
            plotter->replot();
            break;
    }

    return 0;
}

/**Sets the thicknesses of the curves.*/
int plot::setThickness(int thickness, int curve_id)
{
    //set curve thickness according to user choice
    pen[curve_id]->setWidth(thickness+1);
    curve[curve_id]->setPen(*pen[curve_id]);
    plotter->replot();

    return 0;
}

/**Hides/unhides curves.*/
int plot::setVisible(bool visible, int curve_id)
{
    if(visible)
        curve[curve_id]->attach(plotter);

    else
        curve[curve_id]->detach();

    plotter->replot();
    return 0;
}

/**Resets the zoom.*/
int plot::resetZoom()
{
    zoomer->zoom(0);
    return 0;
}

/**Reenables autoscale/prevents the user from zooming out.*/
void plot::on_zoomer_zoomed()
{
    //no more zoom
    if(zoomer->zoomRectIndex() == 0)
    {
        //reactivate autoscale
        plotter->setAxisAutoScale(QwtPlot::xBottom,true);
        plotter->setAxisAutoScale(QwtPlot::yLeft,true);
        plotter->setAxisAutoScale(QwtPlot::yRight,true);

        //adjust plotting section
        plotter->setAxisScale(QwtPlot::xBottom,plot_begin,plot_end);

        plotter->replot();
    }
}

/**Sets curve titles.*/
int plot::setCurveTitle(const QString title, int curve_id)
{
    curve[curve_id]->setTitle(title);
    return 0;
}

/**Sets y axis title.*/
int plot::setYaxisTitle(const QString title)
{
    QwtText label(title);
    QFont font("Arial",11,QFont::Bold);
    label.setFont(font);
    plotter->setAxisTitle(QwtPlot::yLeft, label);
    return 0;
}

/**Rescales the plotter if spinboxes are used.*/
int plot::rescale(int beginning, int end)
{
    plotter->setAxisScale(QwtPlot::xBottom,beginning,end);
    plotter->replot();
    plot_begin = beginning;
    plot_end = end;
    zoomer->zoom(0);
    return 0;
}

/**Sets the time scale to utc values.*/
int plot::setTimeScale()
{
    scaledraw = new TimeScaleDraw;
    plotter->setAxisScaleDraw(QwtPlot::xBottom, scaledraw);
    plotter->setAxisLabelRotation(QwtPlot::xBottom, -30);
    plotter->setAxisLabelAlignment(QwtPlot::xBottom, Qt::AlignLeft | Qt::AlignBottom);
    return 0;
}

/**Add additional y-axis on the right side of the plot canvas.*/
int plot::setAxis(QString name, int index)
{
    plotter->enableAxis(QwtPlot::yRight);
    QwtText label(name);
    QFont font("Arial",11,QFont::Bold);
    label.setFont(font);
    plotter->setAxisTitle(QwtPlot::yRight, label);
    curve[index]->setYAxis(QwtPlot::yRight);
    return 0;
}

/**Update the plot picker to display the current utc values.*/
int plot::updatePlotPicker(QStringList *list)
{
    picker->updateList(list);
    scaledraw->updateList(list);
    return 0;
}

/**Set the xaxis title.*/
int plot::setXaxisTitle(const QString title)
{
    QwtText label(title);
    QFont font("Arial",11,QFont::Bold);
    label.setFont(font);
    plotter->setAxisTitle(QwtPlot::xBottom, label);
    return 0;
}

/**Gets the xvalue where the mouse points.*/
double plot::getXValue()
{
    return picker->plot()->canvasMap(QwtPlot::xBottom).invTransform(picker->trackerPosition().x());
}
