#ifndef PLOT_H
#define PLOT_H

#include <qmath.h>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_grid.h>
#include <qwt_plot_zoomer.h>
#include <qwt_plot_picker.h>
#include <qwt_legend.h>
#include <qwt_scale_draw.h>
#include <qwt_symbol.h>
#include <data.h>
#include <QMessageBox>
#include <QObject>
#include <qwt_text.h>
#include <qwt_plot_panner.h>
#include <qwt_scale_widget.h>
#include <qwt_series_data.h>

#include <QDebug>

#define CURVES_MAX 9

/**The TimeScaleDraw class provides the user with utc data information.
   It derives from QwtScaleDraw and reimplements the label function,
   allowing the application to display utc times as x-values.*/

class TimeScaleDraw: public QwtScaleDraw
{
public:
    TimeScaleDraw()
    {
        list = NULL;
    }

    QStringList *list;

    /**Return a time label.*/
    virtual QwtText label(double v) const
    {
        if(list != NULL)
        {
            if(int(v) < list->count() && int(v) >= 0)
            {
                QString string = list->at(int(v));
                return string.remove(0,string.indexOf(" ")+1);
            }

            else
                return QwtText("");
        }

        else
            return QwtText("");
    }

    /**Update the time list.*/
    void updateList(QStringList *liste)
    {
        list = liste;
    }
};

/**The CustomPicker class enables the user to see the exact utc time when hovering over the plots.
   It derives from QwtPlotPicker and reimplements the trackerText function to show utc times.*/

class CustomPicker: public QwtPlotPicker
{
public:
    CustomPicker(QwtPlotCanvas* parent, bool var): QwtPlotPicker(parent)
    {
        setRubberBand(QwtPlotPicker::CrossRubberBand);
        setTrackerPen(QColor(Qt::black));
        setTrackerMode(QwtPlotPicker::AlwaysOn);

        list = NULL;
        duo = var;
    }

    /**Return custom tracker text.*/
    virtual QwtText trackerText(const QPoint &pos) const
    {
        if(list != NULL)
        {
            QwtText text = QwtPlotPicker::trackerText(pos);
            QString string = text.text();
            string.remove(0,string.indexOf(",")+1);
            string.append(" ");

            QString string0 = text.text();
            string0.remove(string0.indexOf(","),string.length());
            if(int(string0.toDouble()) < list->length() && int(string0.toDouble()) >= 0)
                string.append(list->at(int(string0.toDouble())));

            //add course values
            if(duo)
            {
                double y2 = plot()->canvasMap(QwtPlot::yRight).invTransform(pos.y());
                string.append(" ");
                string.append(QString::number(y2));
                string.append("");
            }

            return string;
        }

        return QwtText("");
    }

    /**Update the time list.*/
    void updateList(QStringList *liste)
    {
        list = liste;
    }

    QStringList *list;
    bool duo;
};

/**The plot class implements the plotter widgets. The data is visualized through this widget.*/

class plot : public QObject
{
    Q_OBJECT

    public:
        plot();
        plot(QwtPlot *plotter_ui, int curves, bool colored, QString yaxis_title,
             QString curve_title0, QString curve_title1, QString curve_title2, bool duo);

        int setData(double *xdata, double *ydata, int curve_id, int length);

        int setColor(int color, int curve_id);
        int setThickness(int thickness, int curve_id);
        int setVisible(bool visible, int curve_id);
        int setCurveTitle(const QString title, int curve_id);
        int setYaxisTitle(const QString title);
        int setXaxisTitle(const QString title);
        int setTimeScale();
        int setAxis(QString name, int index);
        int updatePlotPicker(QStringList *list);
        double getXValue();

        int resetZoom();
        int rescale(int beginning, int end);

        /**Actual plotter widget.*/
        QwtPlot *plotter;
        /**Grid for better visualisation.*/
        QwtPlotGrid *grid;
        /**Enables the user to zoom the plot.*/
        QwtPlotZoomer *zoomer;
        /**Enables the user to pan the plot.*/
        QwtPlotPanner *panner;
        /**Enables the user to get exact information of the plots.*/
        CustomPicker *picker;
        /**Holds the plot legend.*/
        QwtLegend *legend;
        /**Contains the individual curves.*/
        QwtPlotCurve *curve[CURVES_MAX];
        /**Contains the individual pens for drawing.*/
        QPen *pen[CURVES_MAX];
        /**Instance of the timescale.*/
        TimeScaleDraw *scaledraw;

        /**Length of the data.*/
        int length;
        /**Plotting begins from this value.*/
        int plot_begin;
        /**Plotting ends with this value.*/
        int plot_end;

    private slots:
        void on_zoomer_zoomed();
};

#endif // PLOT_H
