#include "thumbnailviewer.h"

thumbnailviewer::thumbnailviewer(QObject *parent ,QGraphicsView *gui_view)
{
    main = parent;
    view = gui_view;
    view->installEventFilter(this);
    imageShown = false;
    scene = new QGraphicsScene;
    jumped = false;
    imageScaling = new QFutureWatcher<ScaledImageInfo>(this);

    connect (imageScaling, SIGNAL(resultReadyAt(int)), SLOT(showImage(int)) );
    connect (view->horizontalScrollBar(), SIGNAL(valueChanged(int)), SLOT(updateView(int)));
    connect(this,SIGNAL(closePicture()),main,SLOT(closePicture()));
}

/** Destroys the thumbnailviewer and closes running threads that load thumbnails. */
thumbnailviewer::~thumbnailviewer(){
    if(imageScaling->isRunning())
        imageScaling->cancel();
    delete scene;
    delete imageScaling;
}

/** Scales the images down to 72x72 px. */
ScaledImageInfo Scale(QString fileName)
{
    ScaledImageInfo result;
    QImage pic(fileName);
    result.scaledImage = pic.scaled(72,72,Qt::IgnoreAspectRatio,Qt::FastTransformation);
    result.scaledImagePath = fileName;
    return result;
}

/** Loads the images from the DCIM folder of the currently loaded data set. */
void thumbnailviewer::loadImages(QDir *directory){
    scene->clear();
    loadedList.clear();
    highlight = new HighlightEffect();

    dir = directory;

    QStringList filter;

    filter << "*.jpg";

    fileList = dir->entryList(filter,QDir::Files);

    for (int i=0;i<fileList.count();++i)
    {
        fileList[i] = dir->path()+QString("/")+fileList.at(i);
    }

    view->setSceneRect(-5,0,fileList.count()*75-15,100);
    //gives the first images to the FutureWatcher to scale them down and display them
    this->updateView(view->horizontalScrollBar()->sliderPosition());
}

/**Loads the thumbnails of the current scrollbar position.*/
void thumbnailviewer::updateView(int pos){
    if(imageScaling->isRunning())
        imageScaling->cancel();
    QStringList subList;
    int sliderPos = pos/75;
    int file = 0;
    while(file*75 < view->width()+75 && sliderPos+file<fileList.count()){
        //don't load thumbnail if it's already shown
        if(!loadedList.contains(fileList.at((pos/75)+file))){
           subList.insert(file,fileList.at((pos/75)+file));
        }
        file++;
    }
    if(!subList.isEmpty()){
        imageScaling->setFuture(QtConcurrent::mapped(subList,Scale));
    }
}

/** Displays the thumbnails on the QGraphicsView. */
void thumbnailviewer::showImage(int num){
    ScaledImageInfo result = imageScaling->resultAt(num);
    QPixmap pic = QPixmap::fromImage(result.scaledImage);
    ProxyWidget *widget = new ProxyWidget(this,result.scaledImagePath, &imageShown);
    QLabel *label = new QLabel;

    //add timestamp to the thumbnail
    QString time = result.scaledImagePath;
    time.remove(0,result.scaledImagePath.length()-10);
    time.truncate(6);
    time.insert(4,":");
    time.insert(2,":");
    QGraphicsSimpleTextItem *timeLabel = scene->addSimpleText(time.toStdString().c_str());
    timeLabel->setPos(fileList.indexOf(result.scaledImagePath)*75+10,80);

    label->setPixmap(pic);
    widget->setWidget(label);
    widget->setPos(fileList.indexOf(result.scaledImagePath)*75,5);
    loadedList.insert(0,result.scaledImagePath);
    scene->addItem(widget);

    view->setScene(scene);

    if(jumped && widget->x()==jumpedPosition){
        widget->setGraphicsEffect(highlight);
        jumped = false;
    }
}

/** Moves the slider in the GraphicsView to the picture of the given time. Is called when
  * the user doubleclicks on the plot. */
void thumbnailviewer::jumpToPicture(QString timeString){
    //return if no pictures were found in the DCIM folder
    if(fileList.isEmpty())
        return;
    //convert timeString into the picture's name
    timeString.remove(timeString.indexOf("-"),1);
    timeString.remove(timeString.indexOf("-"),1);
    timeString.remove(timeString.indexOf("-"),1);
    timeString.insert(timeString.indexOf(" "),"_");
    timeString.remove(timeString.indexOf(" "),1);
    timeString.remove(timeString.length()-3,3);
    timeString.remove(timeString.indexOf(":"),1);
    timeString.remove(timeString.indexOf(":"),1);

    QString sub = timeString.right(6);
    timeString.truncate(timeString.length()-6);

    QTime time = QTime::fromString(sub,"hhmmss");
    //jump to the last picture, if timeString is later than the last taken picture of the data set
    QString last = fileList.last().right(10);
    last.truncate(last.length()-4);
    if(time.toString("hhmmss").toInt()>last.toInt())
        view->horizontalScrollBar()->setSliderPosition(fileList.length()*75);
    else{
        timeString.prepend(dir->absolutePath()+"/iTL_");
        //find the next picture taken after timeString
        while(!fileList.contains(timeString+sub+".jpg")){
           time = time.addSecs(1);
           sub = time.toString("hhmmss");
        }
        timeString.append(sub+".jpg");

        view->horizontalScrollBar()->setSliderPosition((fileList.indexOf(timeString)+1)*75-view->width()/2);
    }
    highlight->setEnabled(true);
    if(loadedList.contains(timeString)){
        QGraphicsItem *item = this->scene->itemAt(((fileList.indexOf(timeString))*75)+5,10);
        item->setGraphicsEffect(highlight);
    }
    else{
        jumped = true;
        jumpedPosition = fileList.indexOf(timeString)*75;
    }
}

/** Closes the enlarged picture view when the mouse leaves the QGraphicsView. */
bool thumbnailviewer::eventFilter(QObject *obj, QEvent *event)
{
    if(obj == view && event->type() == QGraphicsSceneHoverEvent::Leave){
        if(imageShown){
            imageShown = false;
            emit closePicture();
        }
    }
    if(obj == view && event->type() == QEvent::KeyPress){
        return true;
    }
    return false;
}

/** Creates a ProxyWidget for targetFile. */
ProxyWidget::ProxyWidget(thumbnailviewer *parent, QString targetFile, bool *shown)
 : QGraphicsProxyWidget()
{
    viewer = parent;
    file = targetFile;
    imageShown = shown;
    connect(this,SIGNAL(pictureRequest(QString)),viewer->main,SLOT(showPicture(QString)));
    connect(this,SIGNAL(closeRequest()),viewer->main,SLOT(closePicture()));
    connect(this,SIGNAL(highlightSignal(QString)),viewer->main,SLOT(highlighted(QString)));
}

/** Enlarges the clicked thumbnail by loading the original file to a hidden label. */
void ProxyWidget::mousePressEvent(QGraphicsSceneMouseEvent *event){
  if(event->button() == Qt::LeftButton)
  {
     //shows enlarged Image view
     if(!*imageShown)
     {
        *imageShown = true;
        emit pictureRequest(file);
        this->setOpacity(0.9);
      }
      //hides enlarged Image view
     else{
       *imageShown = false;
       this->setOpacity(1);
       emit closeRequest();
     }
  }
}

/** Enlarges the thumbnail over which the mouse hovers when thumbnail enlargement is active. */
void ProxyWidget::hoverEnterEvent(QGraphicsSceneHoverEvent *){
    viewer->highlight->setEnabled(true);
    this->setGraphicsEffect(viewer->highlight);
    emit highlightSignal(file);
    if(*imageShown){
        emit pictureRequest(file);
        this->setOpacity(0.9);
    }
}

/** Removes the highlighting frame by repainting a white frame. */
void ProxyWidget::hoverLeaveEvent(QGraphicsSceneHoverEvent *){
    viewer->highlight->setEnabled(false);
    this->setOpacity(1);
}

/** Creates a HighlightEffect. */
HighlightEffect::HighlightEffect() : QGraphicsEffect(){
}

/** Returns the boundingRect of the highlighting frame. */
QRectF HighlightEffect::boundingRectFor( const QRectF &sourceRect) const
{
   return sourceRect.adjusted(-1.5,-1.5,1.5,1.5);
}

/** Reimplemented from QGraphicsEffect::draw. */
void HighlightEffect::draw(QPainter *painter){
    QPoint offset;
    QPixmap pic = sourcePixmap(Qt::LogicalCoordinates,&offset);

    QRectF bound = boundingRectFor(pic.rect());

    painter->setPen( Qt::NoPen );
    painter->setBrush(QColor(0,100,255,200));
    QPointF p( offset.x()-1.5, offset.y()-1.5);
    bound.moveTopLeft( p );
    painter->drawRoundedRect( bound, 5, 5, Qt::RelativeSize );
    painter->drawPixmap( offset, pic );
}
