/*Copyright (C) 2012 Johannes Hiller, Michael Hoppe, Waldemar Heinrich
* Copyright (C) 2009 The Android Open Source Project
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*   you may not use this file except in compliance with the License.
*   You may obtain a copy of the License at
*
*       http://www.apache.org/licenses/LICENSE-2.0
*
*   Unless required by applicable law or agreed to in writing, software
*   distributed under the License is distributed on an "AS IS" BASIS,
*   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*   See the License for the specific language governing permissions and
*   limitations under the License.
*/


package de.tui.itlogger;



import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;
import com.actionbarsherlock.app.SherlockActivity;
import com.actionbarsherlock.view.MenuItem;


/**
 * This is the main OBD Activity that calls the request of sensor-information, takes the converted integer values from OBD2_Service, corrects the offset and allocates them to a variable.
 */
public class OBD2_Activity extends SherlockActivity {
    // Debugging ?!
    private static final String TAG = "OBD2";
    private static final boolean D = true;
    
    
    // Handler Operationen and PID'S
    public static final int MESSAGE_STATE_CHANGE = 204;
    public static final int Support_1_20 = 0x00;
    public static final int Support_21_40 = 0x20;
    public static final int Support_41_60 = 0x40;
    public static final int Support_61_80 = 0x60;
    public static final int Value_EngineRPM = 0x0C;
    public static final int Value_VehicleSpeed = 0x0D;
    public static final int Value_ThrottlePosition = 0x11;
    public static final int Value_BarometricPressure = 0x33;
    public static final int Value_AmbientAirTemperature = 0x46;
    public static final int Value_ActualEnginePercentTorque = 0x62;
    public static final int Value_EngineReferenceTorque = 0x63;
    public static final int Value_IntakeAirTemperature = 0x0F;
    public static final int Value_VID = 0x02;
    public static final int Startmessage = -1;
    public static final int MESSAGE_WRITE = -2;
    public static final int MESSAGE_DEVICE_NAME = -3;
    public static final int MESSAGE_NOCONNECTION = -4;
    public static final int MESSAGE_OBDERROR = -5;
    public static final int MESSAGE_INITIALOBDERROR = -6;
    public static final int MESSAGE_UNABLE = -7; 
    public static final int MESSAGE_INITIAL_ERROR = -8; 
    public static final int MESSAGE_INITIAL_DONE = -9; 
   
    // Values for DB and GUI
    static int[] OBD_values = new int[8];
    static String VID="";
    // Information about selected sensors
    static boolean[] selected = new boolean[9];
    

    
    static boolean received;
    static boolean[] PID_all = new boolean[128];
    static boolean noreconnect=false;
    static boolean run=false;
   
    
    // Key names received from the OBD2_Service Handler
    public static final String DEVICE_NAME = "device_name";


    // Intent request codes
    private static final int REQUEST_CONNECT_DEVICE_SECURE = 1;
    private static final int REQUEST_ENABLE_BT = 3;


    // Name of the connected device
    private String mConnectedDeviceName = null;

    // String buffer for outgoing messages
    private BluetoothAdapter mBluetoothAdapter = null;
    // Member object for the OBD2 services
    private OBD2_Service mOBD2Service = null;
    
    private startup mstartup;
    private getDATA mgetDATA;


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if(D) Log.e(TAG, "+++ ON CREATE +++");


        // Get local Bluetooth adapter
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        // If the adapter is null, then Bluetooth is not supported
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, "Bluetooth is not available", Toast.LENGTH_LONG).show();
            finish();
            return;
        }
    }
    
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
        	case android.R.id.home:
        		// app icon in action bar clicked; go home
        		Intent intent = new Intent(this, iTloggerActivity.class);
        		intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        		startActivity(intent);
        		break;
        }
        return true;
		}


    @Override
    public void onStart() {
        super.onStart();
        if(D) Log.e(TAG, "++ ON START ++");

		
        
        // If BT is not on, request that it be enabled.
        if (!mBluetoothAdapter.isEnabled()) {
            Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
        // Otherwise, start over OBD
        } else {
            if (mOBD2Service == null) startOBD();
        }
    }

    @Override
    public synchronized void onResume() {
        super.onResume();
        if(D) Log.e(TAG, "+ ON RESUME +");

        // Performing this check in onResume() covers the case in which BT was
        // not enabled during onStart(), so we were paused to enable it...
        // onResume() will be called when ACTION_REQUEST_ENABLE activity returns.
        if (mOBD2Service != null) {
            // Only if the state is STATE_NONE, do we know that we haven't started already
            if (mOBD2Service.getState() == OBD2_Service.STATE_NONE) {
              // Start the OBD2Service
            	mOBD2Service.start();
            }
        }
    }

    private void startOBD() {

                // Send a message using content of the edit text widget
            	Intent intent = new Intent(OBD2_Activity.this, DeviceListActivity.class);
            	startActivityForResult(intent, REQUEST_CONNECT_DEVICE_SECURE); 
    
            	
        // Initialize the OBD2_Service to perform bluetooth connections
        mOBD2Service = new OBD2_Service(this, mHandler);

        
    }

    @Override
    public synchronized void onPause() {
        super.onPause();
        if(D) Log.e(TAG, "- ON PAUSE -");
    }

    @Override
    public void onStop() {
        super.onStop();
        
        if(D) Log.e(TAG, "-- ON STOP --");
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        // Stop the Bluetooth chat services
        if (mOBD2Service != null) mOBD2Service.stop();
        if(D) Log.e(TAG, "--- ON DESTROY ---");
    }


    /**
     * This Handler receives the values from OBD2_Service and causes corresponding event. e.g.: showing Toastmessage or call the correction method to correct the offset and allocates the return value as a variable.   
     */
    private final Handler mHandler = new Handler() {
      

		@Override
        public void handleMessage(Message msg) {
        	
        	int raw=0, value=0, k=0;
        	boolean[] PID_1_20 = new boolean[32];
            boolean[] PID_21_40 = new boolean[32];
            boolean[] PID_41_60 = new boolean[32];
            boolean[] PID_61_80 = new boolean[32];
        	
            switch (msg.what) {
            case MESSAGE_STATE_CHANGE:
                if(D) Log.i(TAG, "MESSAGE_STATE_CHANGE: " + msg.arg1);
                switch (msg.arg1) {
                case OBD2_Service.STATE_CONNECTED:
                   noreconnect=false;
                   run=true;
                	
                	if(iTloggerPreferences.ready == true){
                		mgetDATA = new getDATA();
 	                    mgetDATA.start();
                	}else{
                	
                	 mstartup = new startup();
                     mstartup.start();
                	}
                     
                    break;
                case OBD2_Service.STATE_CONNECTING:
                	Toast.makeText(getApplicationContext(), getString(R.string.CONNECTING), Toast.LENGTH_SHORT).show();
                	
                    break;

                case OBD2_Service.STATE_NONE:
                	Toast.makeText(getApplicationContext(), getString(R.string.NOTCONNECED) , Toast.LENGTH_SHORT).show();
                   
                    break;
                    
                }
                break;
            
            case MESSAGE_NOCONNECTION:
            	
            	run=false;
            	if (mstartup != null) mstartup=null;
            	if (mgetDATA != null) mgetDATA=null;
            	
            	break;
            	
            case MESSAGE_UNABLE:
            	Toast.makeText(getApplicationContext(), getString(R.string.UNABLE_TO_CONNECT) , Toast.LENGTH_SHORT).show();
               
                break;
            
            
            case MESSAGE_INITIAL_ERROR:
            	Toast.makeText(getApplicationContext(), getString(R.string.INITIAL_ERROR) , Toast.LENGTH_SHORT).show();
               
                break;
                
            case MESSAGE_INITIAL_DONE:
            	Toast.makeText(getApplicationContext(), getString(R.string.INITIAL_DONE) , Toast.LENGTH_SHORT).show();
               
                break;
                
            case Startmessage:
            	
                String message = (String) msg.obj;
                Toast.makeText(getApplicationContext(), message , Toast.LENGTH_SHORT).show();
                received=true;
                
                break;  
            
            case MESSAGE_INITIALOBDERROR:
            
            	
                Toast.makeText(getApplicationContext(), getString(R.string.OBDERROR) , Toast.LENGTH_SHORT).show();
                noreconnect=true;
                iTloggerPreferences.ready = false;
		        if (mOBD2Service != null) mOBD2Service.stop();
               
                run=false;
                iTloggerPreferences.ready=false;

                break;  
                
           
            case Support_1_20:
            	// receive array-part from OBD2_Service
            	k=0;
            	PID_1_20 =  (boolean[]) msg.obj;
            	for(int i=0; i<32; i++){
            		// copy the part into PID_all support-list 
            		PID_all[i]=PID_1_20[k];
            		k++;
            	}

            	// set received to true 
                received=true;
                
                break;     
            
            case Support_21_40:
            	
            	PID_21_40 =  (boolean[]) msg.obj;
            	k=0;
            	for(int i=32; i<64; i++){
            		PID_all[i]=PID_21_40[k];
            		k++;
            	}
            	
                received=true;
                
                break; 
            
            case Support_41_60:
            	
            	PID_41_60 =  (boolean[]) msg.obj; 
            	k=0;
            	for(int i=64; i<96; i++){
            		PID_all[i]=PID_41_60[k];
            		k++;
            	}

                received=true;
                
                break; 
            
            case Support_61_80:
            	
            	PID_61_80 =  (boolean[]) msg.obj;
            	k=0;
            	for(int i=96; i<128; i++){
            		PID_all[i]=PID_61_80[k];
            		k++;
            	}
            	
                received=true;
                
                break;  
            
            // add a new case and extend the OBD_values array to support new PID's
                
            case Value_EngineRPM:
            	// receive value from OBD2_Service
                raw = (Integer) msg.obj;
                //call for offset-correction (multiply by 1/4, add 0)
                value=correction(1, 4, raw, 0);
                // save as variable
                OBD_values[0]=value;
                // set received to true 
                received=true;
                
                break;
                
            case Value_VehicleSpeed:
            	
                raw = (Integer) msg.obj;
                OBD_values[1]=raw;
                received=true;
                
                break;
                
            case Value_ThrottlePosition:
            	
                raw = (Integer) msg.obj;
                value=correction(100, 255, raw, 0);
                OBD_values[2]=value;
                received=true;
                
                break;
                
            case Value_BarometricPressure:
            	
                raw = (Integer) msg.obj;
                OBD_values[3]=raw;
                received=true;
                
                break;
                
            case Value_AmbientAirTemperature:
            	
                raw = (Integer) msg.obj;
                value=correction(1, 1, raw, -40);
                OBD_values[4]=value;
                received=true;
                
                break;
                
            case Value_ActualEnginePercentTorque:
            	
                raw = (Integer) msg.obj;
                value=correction(1, 1, raw, -125);
                OBD_values[5]=value;
                received=true;
                
                break;
                
            case Value_EngineReferenceTorque:
            	
                raw = (Integer) msg.obj;
                OBD_values[6]=raw;
                received=true;
                
                break;
                
           
                
            case Value_IntakeAirTemperature:
            	
                raw = (Integer) msg.obj;
                value=correction(1, 1, raw, -40);
                OBD_values[7]=value;
                received=true;
                
                break;
                    
            case MESSAGE_DEVICE_NAME:
                // save the connected device's name
                mConnectedDeviceName = msg.getData().getString(DEVICE_NAME);
                Toast.makeText(getApplicationContext(), getString(R.string.CONNECTED_TO)+" "
                               + mConnectedDeviceName, Toast.LENGTH_SHORT).show();
                break;
            
            }
        }
    };

    /**
     * This this method is called to correct the offset of a received value.
     */
    public int correction(int m,int a, int x, int n){
    	double value=0;
    	value=((m*x)/a)+n;
    	return (int) value;
    }
    
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if(D) Log.d(TAG, "onActivityResult " + resultCode);
        switch (requestCode) {
        case REQUEST_CONNECT_DEVICE_SECURE:
            // When DeviceListActivity returns with a device to connect
            if (resultCode == Activity.RESULT_OK) {
                connectDevice(data, true);
            }
            break;
        
        case REQUEST_ENABLE_BT:
            // When the request to enable Bluetooth returns
            if (resultCode == Activity.RESULT_OK) {
                // Bluetooth is now enabled, so set up a chat session
                startOBD();
            } else {
                // User did not enable Bluetooth or an error occured
                Log.d(TAG, "BT not enabled");
                Toast.makeText(this, R.string.bt_not_enabled_leaving, Toast.LENGTH_SHORT).show();
                finish();
                Intent intent = new Intent(OBD2_Activity.this, iTloggerActivity.class);
            	startActivity(intent);  
            }
        }
    }

    private void connectDevice(Intent data, boolean secure) {
        // Get the device MAC address
        String address = data.getExtras()
            .getString(DeviceListActivity.EXTRA_DEVICE_ADDRESS);
        // Get the BLuetoothDevice object
        BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
        // Attempt to connect to the device
        mOBD2Service.connect(device, secure);
        
        Intent intent = new Intent(OBD2_Activity.this, iTloggerActivity.class);
    	startActivity(intent);  
    }

    void sendMessage(String message){
    	
   	 	byte[] send = new byte[8];
   	 	send = message.getBytes();
   	 	mOBD2Service.write(send);
   	 	received=false;
    	
    }
    
    /**
     * This Thread dose the initialization of the OBD connection. That means it resets the OBD-Dongle and requests the supported PID's.  
     */
    private class startup extends Thread{	
        int _phase=0, errorcount=0;
        
        
        
    	 public void run() {
    		 for(int i=0; i<9;i++){
    	        	selected[i]=true;
    	        	if(i<8){
    	        		OBD_values[i]=-255;	
    	        	}
    	        }
             while((iTloggerPreferences.ready==false) && (_phase<6) && (run == true)){
            	 
             	
             	Log.i(TAG, "_phase:" + _phase);	
             
             
				switch (_phase)
 			{
 				
 				case 0 :
 					
 					try {
						Thread.sleep(4000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
 					
 					// reset Adapter
 					sendMessage("AT Z"+'\r');
                     
                    try {
 						Thread.sleep(4000);
 					} catch (InterruptedException e) {
 						e.printStackTrace();
 					}
                    
                     
                     if(received==true){
                     	_phase++;
                     	errorcount=0;
                     }else{
                    	 if (errorcount == 10){
                    		 _phase=5;
                    	 }
                    	 errorcount++;
                     }
                     
 					break;
 					
 					
 					
 					// request PID's
 				case 1 :
 					
 					// request the value
 					sendMessage("01 00"+'\r');
 					
 					 try {
   						Thread.sleep(2000);
   					} catch (InterruptedException e) {
   						e.printStackTrace();
   					}
 					 
                     if(received==true){
                    	 // go to next phase if received answer and reset error-count
                     	_phase++;
                     	errorcount=0;
                     }else{
                    	 if (errorcount == 10){
                    		 // no answer -> initialization failed
                    		 _phase=5;
                    	 }
                    	 // nothing received -> increment error-count
                    	 errorcount++;
                     }
                     
                    
 					break;
 				case 2 :
 					
 					//only ask for this supported PID's if it is allowed 
 					if(PID_all[31]==true){
 					sendMessage("01 20"+'\r');
 					
 					try {
 						Thread.sleep(2000);
 					} catch (InterruptedException e) {
 						e.printStackTrace();
 					}
                     
                     if(received==true){
                     	_phase++;
                     	errorcount=0;
                     }else{
                    	 if (errorcount == 10){
                    		 _phase=5;
                    	 }
                    	 errorcount++;
                     }
                     
 					}else{
 						//if not supported try next one
                    	 _phase++;
                     }
 					
 					
 					break;
 				case 3 :
 					if(PID_all[63]==true){
 					sendMessage("01 40"+'\r');
 					
 					try {
 						Thread.sleep(2000);
 					} catch (InterruptedException e) {
 						e.printStackTrace();
 					}
                     
                     if(received==true){
                    	 _phase++;
                    	 errorcount=0;
                     }else{
                    	 if (errorcount == 10){
                    		 _phase=5;
                    	 }
                    	 errorcount++;
                     }
                     }else{
                    	 _phase++;
                     }
 					
 					
 					break;
 				case 4 :
 					if(PID_all[95]==true){
 					sendMessage("01 60"+'\r');
 					
 					try {
 						Thread.sleep(2000);
 					} catch (InterruptedException e) {
 						e.printStackTrace();
 					}
                     
 					if(received==true){
 						iTloggerPreferences.ready=true;
 						mgetDATA = new getDATA();
 	                    mgetDATA.start();
 	                    mHandler.obtainMessage(OBD2_Activity.MESSAGE_INITIAL_DONE).sendToTarget();
                     	
                
                     }else{
                    	 if (errorcount == 10){
                    		 _phase=5;
                    	 }
                    	 errorcount++;
                     }
 					}else{ 
                    	 
 					iTloggerPreferences.ready=true;
					mgetDATA = new getDATA();
	                mgetDATA.start();
	                mHandler.obtainMessage(OBD2_Activity.MESSAGE_INITIAL_DONE).sendToTarget();
                      	
                     }
 					
 					 
 					break;
 					
 				case 5 :
 					
 					mHandler.obtainMessage(OBD2_Activity.MESSAGE_INITIAL_ERROR).sendToTarget();
 		            noreconnect=true;
 		            if (mOBD2Service != null) mOBD2Service.stop();
 		            
 		           _phase++;
 		            
 					break;
 					
 				
 			}
             
             }
    	 
    	 }
    	};
    	
    	/**
         * This Thread requests the selected Values periodical in a fix period. Values with a higher sloth will be requested on each 5th period.
         */
    	private class getDATA extends Thread{
    		boolean first=true;
    		boolean start=false;
    		 public void run() {
    			 int count=0;
    			 
    			
    		for(int i=0; i<9; i++){
    			if(selected[i] == true){
    				start=true;
    			}
    		}
    			 
    			 
    		  while((run == true) && (start==true)) {
    			  
    		   if ((first == true) && (selected[8] == true)){
    			   
    			   // request VIN only on first start
    			   sendMessage("09 02"+'\r');
					
					try {
						Thread.sleep(2000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					
					first =false;
    			   
    		   }
    			  
    		   // add new requests
    		   
    		   // only request if sensor is avalible and selected
    		   if((PID_all[Value_EngineRPM-1] == true) && (selected[0] == true)){
    			  sendMessage("01 0"+Integer.toHexString(Value_EngineRPM)+'\r');
    			
    			  		// sleep for a fix delay -> don't stress the dongle too much
    			  		try {
    			  			Thread.sleep(200);
    			  		} catch (InterruptedException e) {
    			  			e.printStackTrace();
    			  		}
    		   }

    		   
    		   
    		   if((PID_all[Value_VehicleSpeed-1] == true) && (selected[1] == true)){
    			   sendMessage("01 0"+Integer.toHexString(Value_VehicleSpeed)+'\r');
   			
    			   		try {
    			   			Thread.sleep(200);
    			   		} catch (InterruptedException e) {
    			   			e.printStackTrace();
    			   		}
    		 }
    		   if((PID_all[Value_ThrottlePosition-1] == true) && (selected[2] == true)){
    			   sendMessage("01 "+Integer.toHexString(Value_ThrottlePosition)+'\r');
   			
    			   		try {
    			   			Thread.sleep(200);
    			   		} catch (InterruptedException e) {
    			   			e.printStackTrace();
    			   		}
    		 }
    		   
    		   if((PID_all[Value_ActualEnginePercentTorque-1] == true) && (selected[5] == true)){
    			   sendMessage("01 "+Integer.toHexString(Value_ActualEnginePercentTorque)+'\r');
   			
    			   		try {
    			   			Thread.sleep(200);
    			   		} catch (InterruptedException e) {
    			   			e.printStackTrace();
    			   		}
    		 }
    		   if((PID_all[Value_EngineReferenceTorque-1] == true) && (selected[6] == true)){
    			   sendMessage("01 "+Integer.toHexString(Value_EngineReferenceTorque)+'\r');
   			
    			   		try {
    			   			Thread.sleep(200);
    			   		} catch (InterruptedException e) {
    			   			e.printStackTrace();
    			   		}
    		 }
    		   count++;
    		   
    		   // Abfrage sich langsamer ndernder Werte
    		   if(count>5){
    			   
    			   count=0;
    			   
           		   if((PID_all[Value_AmbientAirTemperature-1] == true)&& (selected[4] == true)){
           			   sendMessage("01 "+Integer.toHexString(Value_AmbientAirTemperature)+'\r');
          			
           			   		try {
           			   			Thread.sleep(200);
           			   		} catch (InterruptedException e) {
           			   			e.printStackTrace();
           			   		}
           		 }
           		if((PID_all[Value_BarometricPressure-1] == true) && (selected[3] == true)){
     			   sendMessage("01 "+Integer.toHexString(Value_BarometricPressure)+'\r');
    			
     			   		try {
     			   			Thread.sleep(200);
     			   		} catch (InterruptedException e) {
     			   			e.printStackTrace();
     			   		}
           		 }
           		 if((PID_all[Value_IntakeAirTemperature-1] == true) && (selected[7] == true)){
           			   sendMessage("01 0"+Integer.toHexString(Value_IntakeAirTemperature)+'\r');
          			
           			   		try {
           			   			Thread.sleep(200);
           			   		} catch (InterruptedException e) {
           			   			e.printStackTrace();
           			   		}
           		 }
    			   
    		   }

    		  }
    		 }
    		};
    		
    		  
}
